#!/usr/bin/env python3

# Copyright (C) @Misko_2083 2019

import gi
gi.require_version("Gtk", "3.0")
gi.require_version("Gdk", "3.0")
gi.require_version('MatePanelApplet', '4.0')
from gi.repository import Gtk, Gdk
from gi.repository import MatePanelApplet
from gi.repository.GdkPixbuf import Pixbuf
import os
import shlex
import subprocess
import setproctitle

# Rename the process
setproctitle.setproctitle('mate-jgmenu')

class JgMenu(MatePanelApplet.Applet):
    def __init__(self, applet):
        self.applet_fill(applet)

    def execute(self, command):
        subprocess.Popen(shlex.split(command),
                         stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

    def applet_fill(self, applet):
         # you can use this path with gio/gsettings
        settings_path = applet.get_preferences_path()
        applet.set_border_width(0)
        self.button = Gtk.Button(border_width=0)
        try:
           pixbuf = Gtk.IconTheme.get_default().load_icon("start-here", applet.get_size(), 0)
           button_icon = Gtk.Image.new_from_pixbuf(pixbuf)
        except:
           button_icon = Gtk.Label("jgmenu")
        self.button.add(button_icon)
        # Disable label to show icon
#        self.button.set_label("jgmenu")
        self.button.connect("clicked", self.on_toggle, self.button, applet)
        self.applet = applet
        self.applet.add(self.button)
        toplevel = self.button.get_toplevel()
        self.applet.show_all()

    def on_toggle(self, button, applet, data=None):
        # make sure we're in ipc mode - won't touch jgmenurc if already in ipc
        self.execute("jgmenu_run config -s {} -k position_mode -v ipc".format(os.environ["HOME"] + '/.config/jgmenu/jgmenurc'))

        rect = button.get_allocation()
        main_window = self.button.get_toplevel()
        [val, button_x1, button_y1] = main_window.get_window().get_origin()
        button_x2 = button_x1 + rect.width
        button_y2 = button_y1 + rect.height
        button_y1=0
        orientation = self.applet.get_orient()

        # Cheeky hack to watch a few variables
        f = open("/home/johan/mate-temp", 'w')
        f.write("x={} ".format(button_x2))
        f.write("y={} ".format(button_y2))
        f.write("\n")
        f.close()

        os.environ["TINT2_BUTTON_PANEL_X1"] = "0"
        os.environ["TINT2_BUTTON_PANEL_Y1"] = "0"
        os.environ["TINT2_BUTTON_PANEL_X2"] = "1024"
        os.environ["TINT2_BUTTON_PANEL_Y2"] = "{}".format(button_y2)
        os.environ["TINT2_BUTTON_ALIGNED_X1"] = "{}".format(button_x1)
        os.environ["TINT2_BUTTON_ALIGNED_Y1"] = "{}".format(button_y2)
        os.environ["TINT2_BUTTON_ALIGNED_X2"] = "{}".format(button_x2)
        os.environ["TINT2_BUTTON_ALIGNED_Y2"] = "{}".format(button_y2)
        self.execute("jgmenu_run")

def applet_factory(applet, iid, data):
    if iid != "JgMenuApplet":
        return False
    JgMenu(applet)
    return True

if __name__ == "__main__":
    MatePanelApplet.Applet.factory_main("JgMenuAppletFactory", True,
                                    MatePanelApplet.Applet.__gtype__,
                                    applet_factory, None)



#!/usr/bin/env python3

"""
This is a quick hack to give the Unity launcher an invisible and short-lived
"override_redirect=False" window. This is necessary in order to
  - stop the launcher-button flashing for several seconds;
  - enable the launcher-button to be clicked again.

If jgmenu's "override_redirect" window attribute is set to "True" in
x11-ui.c, a new icon will appear in the launcher which we don't want.

If anyone needs this for anything other than Ubuntu's Unity, I'll consider
re-writing it in C to avoid the gtk-3.0 dependency.
"""

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import GLib

def q():
    Gtk.main_quit()
    return False

def timer(widget, data=None):
    GLib.timeout_add(250, q)

window = Gtk.Window()
window.set_opacity(0)
window.set_default_size(0,0)
window.connect("show", timer)
window.show_all()
Gtk.main()



#!/usr/bin/env python3

# Copyright (C) @Misko_2083 2019
# Copyright (C) Johan Malm 2019

""" Parse gtk theme and set some key/value pairs in jgmenurc """

import os
import sys
import shlex
import subprocess
try:
    import gi
except ImportError:
    print("[gtktheme] fatal: require python3-gi")
    sys.exit(1)

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

def run(command):
    """ run a command """
    proc = subprocess.Popen(shlex.split(command))
    proc.wait()

def fmt(s):
    """ ensure string is at least two characters long """
    if len(s) == 0:
        return "00"
    elif len(s) == 1:
        return "0{}".format(s)
    return s

def rgb2hex(line):
    """ convert rgb values to a 6-digit hex string """
    s = line.split("rgb(")
    rgb = s[-1].replace(");", "").split(",")
    r = hex(int(rgb[0]))[2:]
    g = hex(int(rgb[1]))[2:]
    b = hex(int(rgb[2]))[2:]
    return "{}{}{}".format(fmt(r), fmt(g), fmt(b))

def setconfig(key, value):
    """ set key/value pain in ~/.config/jgmenu/jgmenurc """
    filename = os.environ["HOME"] + "/.config/jgmenu/jgmenurc"
    value = "#{} 100".format(value)
    print("[gtktheme] {} = {}".format(key, value))
    cmd = "jgmenu_run config -s {} -k {} -v '{}'".format(filename, key, value)
    run(cmd)

def process_line(line):
    """ process one line """
    if "background-color" in line:
        setconfig("color_menu_bg", rgb2hex(line))
        setconfig("color_title_bg", rgb2hex(line))
        setconfig("color_title_border", rgb2hex(line))

def cache(themename):
    """ save the theme-name to XDG_CACHE_HOME/jgmenu/.last-gtktheme or 
        ~/.cache/jgmenu/.last-gtktheme if it doesn't exist """
    print("themename={}".format(themename))
    home = os.environ["HOME"]
    directory = os.getenv("XDG_CACHE_HOME", home + "/.cache") + "/jgmenu"
    if not os.path.exists(directory):
        os.mkdir(directory)
    f = open(directory + "/.last-gtktheme", "w")
    f.write(themename)
    f.close()

def main():
    """ main """
    gset = Gtk.Settings.get_default()
    themename = gset.get_property("gtk-theme-name")
    cache(themename)
    prefdark = gset.get_property("gtk-application-prefer-dark-theme")
    css = Gtk.CssProvider.get_named(themename).to_string()
#    print(css)
#    exit(1)
    lines = css.split("\n")

    # parse some @define-color lines
    for line in lines:
        if "@define-color" not in line:
            break
        if "theme_text_color" in line:
            setconfig("color_norm_fg", rgb2hex(line))
            setconfig("color_title_fg", rgb2hex(line))
        if "theme_selected_bg_color" in line:
            setconfig("color_sel_bg", rgb2hex(line))
        if "theme_selected_fg_color" in line:
            setconfig("color_sel_fg", rgb2hex(line))

    # parse the menu { } section
    inside = False
    for line in lines:
        if "menu {" in line:
            inside = True
            continue
        if inside:
            if "{" in line:
                inside = False
                break
            process_line(line)

if __name__ == '__main__':
    main()

